/**
 * @file app_led.c
 * @brief tuyaos quick start demo
 * @version 1.0
 * @date 2022-02-18
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */

/***********************************************************************
 ** INCLUDE                                                           **
 **********************************************************************/
#include "tal_log.h"
#include "tuya_gpio.h"
#include "tal_thread.h"
#include "tuya_cloud_wifi_defs.h"

#include "tuya_app_led.h"
#include "tuya_tal_2_tkl.h"
#include "tuya_wifi_status.h"
/***********************************************************************
 ** CONSTANT ( MACRO AND ENUM )                                       **
 **********************************************************************/
#define LED_PIN 24

/***********************************************************************
 ** STRUCT                                                            **
 **********************************************************************/

/***********************************************************************
 ** VARIABLE                                                          **
 **********************************************************************/
static UINT8_T cur_led_status = LED_OFF;

THREAD_HANDLE ty_blink_task_handle;

/***********************************************************************
 ** FUNCTON                                                           **
 **********************************************************************/

/**
* @brief led init
*
* @param[in] none
* @return none
*/
void app_led_init(void)
{
    TUYA_GPIO_BASE_CFG_T led_cfg = {
        .mode = TUYA_GPIO_PUSH_PULL,
        .direct = TUYA_GPIO_OUTPUT,
        .level = TUYA_GPIO_LEVEL_HIGH
    };
    tal_gpio_init(LED_PIN, &led_cfg);

    set_led_status(LED_OFF);

    app_led_thread_init();
    return;
}

/**
* @brief set led 
*
* @param[in] led_status: LED_ON: led turn on, LED_OFF: led turn off
* @return none
*/
void set_led_status(LED_STATUS_E led_status)
{
    if (LED_ON == led_status) {
        tal_gpio_write(LED_PIN, TUYA_GPIO_LEVEL_LOW);
    } else {
        tal_gpio_write(LED_PIN, TUYA_GPIO_LEVEL_HIGH);
    }

    cur_led_status = led_status;
    return;
}

/**
* @brief get led status, on or off
*
* @param[in] none
* @return current led status
*/
LED_STATUS_E get_led_status(void)
{
    return cur_led_status;
}

/**
* @brief Blinking LED when WiFi is in a wiring state
*
* @param[in] args: blink interval
* @return none
*/
static void wifi_status_display_task(void* args)
{
    UINT32_T blink_interval = (UINT32_T)(args);
    GW_WIFI_NW_STAT_E network_status;

    while (1) {
        get_wf_gw_nw_status(&network_status);
        if ((network_status>=STAT_UNPROVISION) && (network_status<=STAT_STA_CONN)) {
            tal_gpio_write(LED_PIN, TUYA_GPIO_LEVEL_LOW);
            tal_system_sleep(blink_interval);
            tal_gpio_write(LED_PIN, TUYA_GPIO_LEVEL_HIGH);
            tal_system_sleep(blink_interval);
        } else {
            tal_system_sleep(3000);
        }
    }
}

/**
* @brief init led thread 
*
* @param[in] none
* @return none
*/
void app_led_thread_init(void)
{
    // OPERATE_RET op_ret = OPRT_OK;
    UINT32_T blink_interval = 500;

    THREAD_CFG_T  blink_thread_cfg = {
        .thrdname = "led_task",
        .priority = THREAD_PRIO_6,
        .stackDepth = 1024*2
    };

    tal_thread_create_and_start(&ty_blink_task_handle, \
                                NULL, \
                                NULL, \
                                wifi_status_display_task, \
                                (CONST PVOID_T)blink_interval, \
                                &blink_thread_cfg);
}
