/**
 * @file tuya_device.c
 * @brief tuyaos quick start demo
 * @version 1.0
 * @date 2022-02-18
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */
#define _TUYA_DEVICE_GLOBAL

#include "tuya_iot_config.h"

#include "tal_log.h"
#include "tuya_iot_wifi_api.h"
#include "tuya_hal_system.h"
#include "tuya_cloud_wifi_defs.h"
#include "tuya_iot_com_api.h"
#include "gw_intf.h"

#include "tuya_device.h"
#include "tuya_dp_process.h"
#include "tuya_app_key.h"
#include "tuya_app_led.h"
#include "wf_basic_intf.h"

/***********************************************************************
 ** INCLUDE                                                           **
 **********************************************************************/

/***********************************************************************
 ** CONSTANT ( MACRO AND ENUM )                                       **
 **********************************************************************/

/***********************************************************************
 ** STRUCT                                                            **
 **********************************************************************/

/***********************************************************************
 ** VARIABLE                                                          **
 **********************************************************************/
// THREAD_HANDLE ty_blink_task_handle = NULL;

/***********************************************************************
 ** FUNCTON                                                           **
 **********************************************************************/

/**
* @brief pre_gpio_test gpio test pre-interface, used to prepare for the gpio test
*
* @param[in] none
* @return none
*/
BOOL_T mf_user_pre_gpio_test_cb(VOID)
{
    return TRUE;
}

/**
* @brief erase application data from flash
*
* @param[in] none
* @return none
*/
STATIC VOID hw_reset_flash_data(VOID)
{
    /* erase application data from flash */
    return;
}

/**
* @brief configuring the write callback interface
*
* @param[in] none
* @return none
*/
VOID mf_user_callback(VOID)
{
    hw_reset_flash_data();
    return;
}

/**
* @brief configure to enter the production test callback interface
*
* @return none
*/
VOID mf_user_enter_callback(VOID)
{
    return;
}

#define BOOL_CMD 0xFF03
#define MAX_BUF_LEN 128
extern void xr806_switch_etf(void);
/**
* @brief Finished Product test callbacks
*
* @param[in] none
* @return OPRT_OK
*/
OPERATE_RET mf_user_product_test_cb(USHORT_T cmd, UCHAR_T *data, UINT_T len, OUT UCHAR_T **ret_data, OUT USHORT_T *ret_len)
{
    TAL_PR_INFO("mf_user_product_test_cb cmd:0x%x", cmd);
    int ret;
    cJSON* root = cJSON_Parse((const char*)data);
    if (NULL == root) {
        TAL_PR_DEBUG("Json parse error");
        return OPRT_CJSON_PARSE_ERR;
    }

    cJSON *sub_cmd = cJSON_GetObjectItem(root, "testItem");
    if (sub_cmd == NULL) {
        TAL_PR_ERR("testItem get error");
        return OPRT_CJSON_GET_ERR;
    }

    UCHAR_T *buf = malloc(MAX_BUF_LEN);
    memset(buf, 0, MAX_BUF_LEN);
    switch (cmd) {
    case BOOL_CMD:
        ret = strcmp("switchFirmware", sub_cmd->valuestring);
        TAL_PR_NOTICE("cmd string: %s", sub_cmd->valuestring);
        if (ret == 0) {
            sprintf(buf, "{\"ret\":true}");
            tkl_system_sleep(1000);
            if (strcmp(TARGET_PLATFORM, "xr806") == 0)
                xr806_switch_etf();
        } else {
            sprintf(buf, "{\"ret\":false,\"errCode\":500000}");
        }
        break;

    default:
        break;
    }

    *ret_data = buf;

    return OPRT_OK;
}

/**
* @brief application initialization prep work
*
* @param[in] none
* @return none
*/
// extern void fast_start(void);
VOID pre_app_init(VOID)
{
    return;
}

STATIC VOID tuya_rst_inform_cb(VOID)
{
    TAL_PR_NOTICE("#####tuya_rst_inform_cb#####");
    tuya_iot_wf_gw_fast_unactive(GWCM_LOW_POWER, WF_START_SMART_FIRST);
}

/**
* @brief device initialization prep work
*
* @param[in] none
* @return none
*/
VOID pre_device_init(VOID)
{
    TAL_PR_NOTICE("%s", tuya_iot_get_sdk_info()); /* print SDK information */
    TAL_PR_NOTICE("%s:%s", APP_BIN_NAME, DEV_SW_VERSION); /* print the firmware name and version */
    TAL_PR_NOTICE("firmware compiled at %s %s", __DATE__, __TIME__); /* print firmware compilation time */
    TAL_PR_NOTICE("system reset reason:[%s]", tuya_hal_system_get_rst_info()); /* print system reboot causes */

    tal_log_set_manage_attr(TAL_LOG_LEVEL_DEBUG); /* set log level */
#if ENABLE_APP_RST_CFG
    tuya_set_cnt_rst_mode(3, tuya_rst_inform_cb);
#endif
    return;
}

/**
* @brief application initialization interface
*
* @param[in] none
* @return none
*/
VOID app_init(VOID)
{
#if ENABLE_APP_RST_CFG
    tuya_cnt_rst_judge();
#endif
    return;
}

/**
* @brief report all dp status
*
* @param[in] none
* @return none
*/
VOID hw_report_all_dp_status(VOID)
{
    //report all dp status
    update_all_dp();
    return;
}

/**
* @brief device activation status change callback
*
* @param[in] status: current status
* @return none
*/
VOID status_changed_cb(IN CONST GW_STATUS_E status)
{
    TAL_PR_NOTICE("status_changed_cb is status:%d", status);

    if(GW_NORMAL == status) {
        hw_report_all_dp_status();
    }else if(GW_RESET == status) {
        TAL_PR_NOTICE("status is GW_RESET");
    }

    return;
}

/**
* @brief firmware download finish result callback
*
* @param[in] fw: firmware info
* @param[in] download_result: 0 means download succes. other means fail
* @param[in] pri_data: private data
* @return none
*/
OPERATE_RET ty_upgrade_notify_cb(IN CONST FW_UG_S *fw, IN CONST INT_T download_result, IN PVOID_T pri_data)
{
    TAL_PR_DEBUG("download  Finish");
    TAL_PR_DEBUG("download_result:%d", download_result);

    return 0;
}

/**
* @brief firmware download content process callback
*
* @param[in] fw: firmware info
* @param[in] total_len: firmware total size
* @param[in] offset: offset of this download package
* @param[in] data && len: this download package
* @param[out] remain_len: the size left to process in next cb
* @param[in] pri_data: private data
* @return OPRT_OK: success  Other: fail
*/
OPERATE_RET get_file_data_cb(IN CONST FW_UG_S *fw, IN CONST UINT_T total_len, IN CONST UINT_T offset,
                                IN CONST BYTE_T *data, IN CONST UINT_T len, OUT UINT_T *remain_len, IN PVOID_T pri_data)
{
    TAL_PR_DEBUG("Rev File Data");
    TAL_PR_DEBUG("Total_len:%d ", total_len);
    TAL_PR_DEBUG("Offset:%d Len:%d", offset, len);

    return OPRT_OK;
}


/**
* @brief ota inform callback
*
* @param[in] fw: firmware info
* @return
*/
INT_T gw_ug_inform_cb(IN CONST FW_UG_S *fw)
{
    TAL_PR_DEBUG("Rev GW Upgrade Info");
    TAL_PR_DEBUG("fw->fw_url:%s", fw->fw_url);
    TAL_PR_DEBUG("fw->sw_ver:%s", fw->sw_ver);
    TAL_PR_DEBUG("fw->file_size:%d", fw->file_size);

    return tuya_iot_upgrade_gw(fw, get_file_data_cb, ty_upgrade_notify_cb, NULL);
}

/**
* @brief called after reset device or app remove device
*
* @param[in] type: gateway reset type
* @return none
* @others reset factory clear flash data
*/
VOID gw_reset_cb(IN CONST GW_RESET_TYPE_E type)
{
    TAL_PR_DEBUG("gw_reset_cb type:%d", type);

    if(GW_REMOTE_RESET_FACTORY != type) {
        TAL_PR_DEBUG("type is GW_REMOTE_RESET_FACTORY");
        return;
    }

    hw_reset_flash_data();

    return;
}

/**
* @brief called after the cloud sends data of type bool, value, enum, string or fault
*
* @param[in] dp: obj dp info
* @return none
*/
VOID dev_obj_dp_cb(IN CONST TY_RECV_OBJ_DP_S *dp)
{
    UCHAR_T i = 0;

    TAL_PR_DEBUG("dp->cid:%s dp->dps_cnt:%d", dp->cid, dp->dps_cnt);

    for(i = 0; i < dp->dps_cnt; i++) {
        deal_dp_proc(&(dp->dps[i]));
        dev_report_dp_json_async(get_gw_cntl()->gw_if.id, dp->dps, dp->dps_cnt);
    }

    return;
}

/**
* @brief called after the cloud sends data of type raw
*
* @param[in] dp: raw dp info
* @return none
*/
VOID dev_raw_dp_cb(IN CONST TY_RECV_RAW_DP_S *dp)
{
    TAL_PR_DEBUG("raw data dpid:%d", dp->dpid);
    TAL_PR_DEBUG("recv len:%d", dp->len);

#if APP_RAW_PRINT_DEBUG
    INT_T i = 0;
    for(i = 0; i < dp->len; i++) {
        TAL_PR_DEBUG_RAW("%02X ", dp->data[i]);
    }
    TAL_PR_DEBUG_RAW("\n");
#endif

    return;
}

/**
* @brief query device dp status
*
* @param[in] dp_qry: query info
* @return none
*/
STATIC VOID dev_dp_query_cb(IN CONST TY_DP_QUERY_S *dp_qry)
{
    TAL_PR_NOTICE("Recv DP Query Cmd");

    hw_report_all_dp_status();

    return;
}

#define GPIO_WAKEUP_NUM TUYA_GPIO_NUM_12
typedef struct {
    TUYA_GPIO_NUM_E     gpio;
    TUYA_GPIO_IRQ_CB    cb;
    void                *args;
    TUYA_GPIO_IRQ_E      evt;
} pin_dev_map_t;

static VOID_T gpio_wake_up_cb(VOID_T *args)
{
    pin_dev_map_t* pinmap = (pin_dev_map_t*)args;

    TAL_PR_NOTICE("-------------------pin_id:%d", pinmap->gpio);
}
/**
* @brief This function is called when the state of the device connection has changed
*
* @param[in] stat: curr network status
* @return none
*/
VOID wf_nw_status_cb(IN CONST GW_WIFI_NW_STAT_E stat)
{
    TAL_PR_NOTICE("wf_nw_status_cb, wifi_status:%d", stat);

    if (STAT_CLOUD_CONN == stat || STAT_AP_CLOUD_CONN == stat) {
        hw_report_all_dp_status();
#if ENABLE_APP_WIFI_LOWPOWER
        TUYA_GPIO_IRQ_T cfg = {
            .arg = NULL,
            .mode = TUYA_GPIO_IRQ_RISE_FALL,
            .cb = &gpio_wake_up_cb,
        };
        tkl_gpio_irq_init(GPIO_WAKEUP_NUM, &cfg);
        tkl_gpio_irq_enable(GPIO_WAKEUP_NUM);

        TUYA_WAKEUP_SOURCE_BASE_CFG_T param = {
            .source = TUYA_WAKEUP_SOURCE_GPIO,
            .wakeup_para.gpio_param.gpio_num = GPIO_WAKEUP_NUM,
            .wakeup_para.gpio_param.level = TUYA_GPIO_LEVEL_LOW,
        };
        //tkl_wakeup_source_set(&param);
#if ENABLE_APP_CPU_DEEP_SLEEP
        tuya_hal_set_cpu_lp_mode(TRUE, TUYA_CPU_DEEP_SLEEP);
#else
        tal_cpu_set_lp_mode(true);
        tuya_wifi_set_lps_dtim(10);
        tal_wifi_lp_enable();
#endif
#endif
    }

    return;
}
//#include "pm/pm.h"
//#include "driver/chip/hal_wakeup.h"

/**
* @brief device initialization interface
*
* @param[in] none
* @return OPRT_OK: success, others: please refer to tuya error code description document
*/
OPERATE_RET device_init(VOID)
{
    OPERATE_RET op_ret = OPRT_OK;

    TY_IOT_CBS_S wf_cbs = {
        status_changed_cb, \
        gw_ug_inform_cb, \
        gw_reset_cb, \
        dev_obj_dp_cb, \
        dev_raw_dp_cb, \
        dev_dp_query_cb, \
        NULL \
    };

    /* tuya IoT framework initialization */
    op_ret = tuya_iot_wf_soc_dev_init_param(GWCM_OLD_PROD, WF_START_SMART_FIRST, &wf_cbs, NULL, PRODECT_ID, DEV_SW_VERSION);
    if(OPRT_OK != op_ret) {
        TAL_PR_ERR("tuya_iot_wf_soc_dev_init_param error,err_num:%d", op_ret);
        return op_ret;
    }

    /* register Wi-Fi connection status change callback function */
    op_ret = tuya_iot_reg_get_wf_nw_stat_cb(wf_nw_status_cb);
    if(OPRT_OK != op_ret) {
        TAL_PR_ERR("tuya_iot_reg_get_wf_nw_stat_cb is error,err_num:%d", op_ret);
        return op_ret;
    }

    app_led_init();
    app_key_init();

    TUYA_GPIO_BASE_CFG_T cfg = {0};
    cfg.direct = TUYA_GPIO_OUTPUT;
    cfg.level = TUYA_GPIO_LEVEL_HIGH;
    cfg.mode = TUYA_GPIO_PUSH_PULL;
    tkl_gpio_init(24+7, &cfg);
    tkl_gpio_write(24+7, TUYA_GPIO_LEVEL_HIGH);

    return op_ret;
}


